@@ -42,14 +42,14 @@
 TodoCategoriesModel::TodoCategoriesModel(QObject *parent)
     : TodoProxyModelBase(MultiMapping, parent), m_categoryRootNode(0)
 {
-    connect(&CategoryManager::instance(), SIGNAL(categoryAdded(const QString&)),
-            this, SLOT(createCategoryNode(const QString&)));
-    connect(&CategoryManager::instance(), SIGNAL(categoryRemoved(const QString&)),
-            this, SLOT(removeCategoryNode(const QString&)));
-    connect(&CategoryManager::instance(), SIGNAL(categoryRenamed(const QString&, const QString&)),
-            this, SLOT(renameCategoryNode(const QString&, const QString&)));
-    connect(&CategoryManager::instance(), SIGNAL(categoryMoved(const QString&, const QString&)),
-            this, SLOT(moveCategoryNode(const QString&, const QString&)));
+    connect(&CategoryManager::instance(), SIGNAL(categoryAdded(QString)),
+            this, SLOT(createCategoryNode(QString)));
+    connect(&CategoryManager::instance(), SIGNAL(categoryRemoved(QString)),
+            this, SLOT(removeCategoryNode(QString)));
+    connect(&CategoryManager::instance(), SIGNAL(categoryRenamed(QString,QString)),
+            this, SLOT(renameCategoryNode(QString,QString)));
+    connect(&CategoryManager::instance(), SIGNAL(categoryMoved(QString,QString)),
+            this, SLOT(moveCategoryNode(QString,QString)));
 }
 
 TodoCategoriesModel::~TodoCategoriesModel()
@@ -73,7 +73,25 @@ void TodoCategoriesModel::onSourceInsertRows(const QModelIndex &sourceIndex, int
                 addChildNode(sourceChildIndex, m_inboxNode);
 
             } else {
-                foreach (const QString &category, categories) {
+                QList<TodoNode*> nodes = m_manager->nodesForSourceIndex(sourceChildIndex);
+                QSet<QString> oldCategories;
+
+                foreach (TodoNode *node, nodes) {
+
+                    TodoNode *categoryNode = node->parent();
+                    if (categoryNode
+                     && categoryNode->data(0, Zanshin::ItemTypeRole).toInt()!=Zanshin::Inbox) {
+                        QString category = categoryNode->data(0, Zanshin::CategoryPathRole).toString();
+                        oldCategories << category;
+                    }
+                }
+
+                QSet<QString> newCategories = QSet<QString>::fromList(categories);
+                QSet<QString> interCategories = newCategories;
+                interCategories.intersect(oldCategories);
+                newCategories-= interCategories;
+
+                foreach (const QString &category, newCategories) {
                     TodoNode *parent = m_categoryMap[category];
                     Q_ASSERT(parent);
                     addChildNode(sourceChildIndex, parent);
@@ -207,7 +225,7 @@ void TodoCategoriesModel::init()
         beginInsertRows(QModelIndex(), 1, 1);
 
         TodoNode *node = new TodoNode;
-        node->setData(i18n("Categories"), 0, Qt::DisplayRole);
+        node->setData(i18n("Contexts"), 0, Qt::DisplayRole);
         node->setData(KIcon("document-multiple"), 0, Qt::DecorationRole);
         node->setRowData(Zanshin::CategoryRoot, Zanshin::ItemTypeRole);
 
@@ -229,7 +247,7 @@ TodoNode *TodoCategoriesModel::createInbox() const
 {
     TodoNode *node = new TodoNode;
 
-    node->setData(i18n("No Category"), 0, Qt::DisplayRole);
+    node->setData(i18n("No Context"), 0, Qt::DisplayRole);
     node->setData(KIcon("mail-folder-inbox"), 0, Qt::DecorationRole);
     node->setRowData(Zanshin::Inbox, Zanshin::ItemTypeRole);
 
